#The following code allows the user to recreate the results obtained with the complementary index

#######
#Step 1: Reading in the data
#######

#First set the working directory
setwd("C:/Users/kvincent/Desktop/MoH PCA project/Kyle's work")
#Read in all the data, call the headers for extra information like min/max values of variables
Data_All = read.csv("Complete Reduced Database Final.csv",header=TRUE)

#Removing the preamble, this is what we work with
Data_sub_All = read.csv("Data Working Version.csv",header=TRUE)
#Data_sub_All contains the former assignment of dissemination areas to local health areas

#Upload the British Columbia Ministry of Health's file
New_LHAs = read.csv("DA_CSD_LHA_2006_Jeet_Rana.csv",header=TRUE)

#The Unused_New_LHAs are those dissemination areas that are not included in our oringial data set and can't be used due to an absence of reported income/census data
#Here are the unused dissemination areas, there are 1,031 of them
Unused_New_LHAs = subset(New_LHAs,New_LHAs[,1] %in% intersect(New_LHAs[,1],Data_sub_All[,2])==FALSE)

#Here are the new assignments for the 6,440 dissemination areas
New_LHAs = subset(New_LHAs,New_LHAs[,1] %in% intersect(New_LHAs[,1],Data_sub_All[,2]))

#Carry the new assignments over to our original census data set
Data_sub_All[,6:7] = New_LHAs[,3:4] #Update the file we work with

#Load libraries, download these from the CRAN website or via installing through the packages tab in R
library(fields)  #For drawing liines in R
library(tseries) #For the Jaques-Bera test in R


#######
#Step 2: The data reduction step
#######

#Working with the necessary data
Data = Data_sub_All[,c(5,8:84)]

Data = subset(Data,select=-c(POP15,POPALL))	#First remove the 2 population measures

num_var = dim(Data)[2]						#Now remove those 17 variables with a median of zero
medians = numeric()
for(k in 1:num_var)
	if(median(Data[,k]) == 0)
		medians = c(medians,k)

Data = Data[,setdiff(1:num_var,medians)]


num_var = dim(Data)[2]						#Now remove the highly correlated variables, 16 variables are removed at this step
spearman.cor = numeric()
for(k in 1:(num_var-1))
	{
	for(j in (k+1):num_var)
		if(cor(Data[,k],Data[,j],method=c("spearman")) > abs(0.8))
			spearman.cor = c(spearman.cor,j)
	}

Data = Data[,setdiff(1:num_var,unique(spearman.cor))]	
	

#######
#Step 3: The principal variable approach (this step may take up to 10 minutes to complete)
#######

Data = scale(Data)			#First scale the data, easiest to work with this

pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0) #The preliminary principal components anlaysis (PCA)
		
plot(pData$sdev^2,typ='h'); abline(a=1,b=0,col="blue");		#Plot the eigenvalues

K = length(which(pData$sdev^2 >= 1))				#Our cutoff is 1, K is the number of pc's to work with
K

loadings(pData)			#The eigenvectors, missing entries are approximately zero

num_var = dim(Data)[2]
pData$scores[1:10,1:num_var]  #The Princiapl Components, only the first 10 entries for each are shown here

#Now the actual principal variable (with K variables) approach: there are a lot of subsets to try here. 
#For now we will implement a search algorithm that McCabe (1984, [Technometrics]) suggested, for finding that subset that satisfies criteria two
#This step is very computationally expensive

keep.pv = numeric()
variables.left=1:num_var

for(k in 1:K)
	{
	maximum = c(0,0)
	for(i in variables.left)
		{
		r.squareds = 0
		for(j in 1:num_var)
			{
			r.squareds = r.squareds+summary(lm(Data[,j] ~ Data[,c(keep.pv,i)]))$r.squared
			}
		if(r.squareds > maximum[1])
			{
			maximum[1] = r.squareds
			maximum[2] = i
			}
		}
	keep.pv = c(keep.pv,maximum[2])
	variables.left = setdiff(variables.left,maximum[2])
	}
	
keep.pv = sort(keep.pv)			#The variables that we keep for the principal variable approach
colnames(Data)[keep.pv]			#With column names to identify them

#######
#Step 4: Now to run the actual PCA on the K variables
#######

Data = Data[,keep.pv]			#The data set for the PCA

#First check the distribution of the data, here we check for outliers in the remaining variables
par(mfrow=c(3,3))
for(k in 1:dim(Data)[2])
	{
	if(k %% 9 ==1)
		{
		X11();
		par(mfrow=c(3,3))
		}
		hist(Data[,k])
	}

pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)		#The PCA
weights.pca.1 = loadings(pData)[,1] #The index values
weights.pca.2 = loadings(pData)[,2] #A second index, may provide some insight into a material vs. social deprivation
#Compare these two to see a potential contrast 
weights.pca.1
weights.pca.2

#######
#Step 5: Obtaining scores for the dissemination areas
#######

Score.complementary = numeric(6440)
for(k in 1:6440)
	Score.complementary[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.complementary = (Score.complementary - mean(Score.complementary)) / sd(Score.complementary)
Score.complementary = 100*Score.complementary + 1000

X11();	
hist(Score.complementary,col="lightblue",breaks=100,xlim=range(550,1450))	#A histogram to see the scores

#Test for Normality, the Jaque Bera test depends on skewness and kurtosis. However, this is a very large sample, therefore the QQ plot is strongly suggested.
jarque.bera.test(Score.complementary)

qqnorm(Score.complementary)				#The QQ-plot


#The 5 dissemination areas that received the highest scores with scores provided, the cutoff of 1310.8 gives the 5 highest scores
Data_sub_All[which(Score.complementary>1310.8),1:7]
Score.complementary[which(Score.complementary>1310.8)]

#The 5 dissemination areas that received the lowest scores with scores provided, the cutoff of 653.1 gives the 5 lowest scores
Data_sub_All[which(Score.complementary<653.1),1:7]
Score.complementary[which(Score.complementary<653.1)]


#######
#Step 6: Obtaining scores for the local health areas
#######

#Append the scores from dissemination areas to the updated data set
Data_Score.complementary = cbind(Data_sub_All[,1:7],Score.complementary)

num_var = dim(Data_Score.complementary)[1]
LHA_Score.complementary = numeric(length(levels(Data_Score.complementary[,7]))) #Will be the LHA scores

jj = 0
for(k in levels(Data_Score.complementary[,7]))
	{
	score.temp = numeric()
	j = 0
	for(i in 1:num_var)
		{
		if(Data_Score.complementary[i,7] == k)
			{
			j = j+1
			score.temp[j] = Data_Score.complementary[i,8]
			}
		}
	jj = jj+1
	LHA_Score.complementary[jj] = mean(score.temp)	
	}

LHA_Score.complementary = cbind(LHA_Score.complementary,levels(Data_Score.complementary[,7])) #The final scores
LHA_Score.complementary #First by ALPHABETICAL order

num_var=dim(LHA_Score.complementary)[1]
scores = as.numeric(LHA_Score.complementary[,1])

for(k in 1:length(scores))
	if(is.finite(scores[k]) == FALSE)
		scores[k] = 0
		
		
LHA_Score.complementary=LHA_Score.complementary[sort(scores,index.return=TRUE)$ix,]
LHA_Score.complementary #Now by ASCENDING order



	
################################	
#Recreating the prototype index
#This is the work based on the prototype index and can be found in Regional Measures (2012)
#The work that is recreated here uses the new assignemts for the dissemination areas
###############################

#Working with the necessary data
Data = Data_sub_All[,c(5,8:84)]

#Data Reduction
Data = subset(Data,select=-c(POP15,POPALL))	#First remove the population measures

num_var = dim(Data)[2]						#Now remove those variables with a median of zero
medians = numeric()
for(k in 1:num_var)
	if(median(Data[,k]) == 0)
		medians = c(medians,k)

Data = Data[,setdiff(1:num_var,medians)]

#The following variables were those that were retained for the principal variable model
prototype = numeric()
prototype[1]=which(colnames(Data)==c("INCMEH"))
prototype[2]=which(colnames(Data)==c("EDNO65"))
prototype[3]=which(colnames(Data)==c("OCCONE"))
prototype[4]=which(colnames(Data)==c("UNEMP"))
prototype[5]=which(colnames(Data)==c("EDCO65"))
prototype[6]=which(colnames(Data)==c("EDCO25"))
prototype[7]=which(colnames(Data)==c("NMTONG"))
prototype[8]=which(colnames(Data)==c("EDHS65"))
prototype[9]=which(colnames(Data)==c("EDCO15"))
prototype[10]=which(colnames(Data)==c("EDHS15"))
prototype[11]=which(colnames(Data)==c("EDAP65"))
prototype[12]=which(colnames(Data)==c("PART15"))
prototype


#Now the loadings from the first pc for the weights
Data = Data[,prototype]
Data = scale(Data)			#Scale the data
pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)
weights.pca.1 = -1*loadings(pData)[,1]
weights.pca.2 = -1*loadings(pData)[,2]
#Compare these two to see a potential contrast in material and social deprivation
weights.pca.1
weights.pca.2

#Now the scores for the 6440 dissemination areas, based on the first index
Score.prototype = numeric(6440)
for(k in 1:6440)
	Score.prototype[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.prototype = (Score.prototype - mean(Score.prototype)) / sd(Score.prototype)
Score.prototype = 100*Score.prototype + 1000

hist(Score.prototype,col="lightblue",breaks=100,xlim=range(550,1450)) 

#Test for Normality
jarque.bera.test(Score.prototype)	

qqnorm(Score.prototype)			#Providing a qq plot


#Now to score over the LHAs
#This step will take up to 2 minutes to complete
#These would be the prototype's final scores based on the updated list of the assignment of dissemination areas to LHAs
Data_Score.prototype = cbind(Data_sub_All[,1:7],Score.prototype)

num_var = dim(Data_Score.prototype)[1]
LHA_Score.prototype = numeric(length(levels(Data_Score.prototype[,7])))

jj = 0
for(k in levels(Data_Score.prototype[,7]))
	{
	score.temp = numeric()
	j = 0
	for(i in 1:num_var)
		{
		if(Data_Score.prototype[i,7] == k)
			{
			j = j+1
			score.temp[j] = Data_Score.prototype[i,8]
			}
		}
	jj = jj+1
	LHA_Score.prototype[jj] = mean(score.temp)	
	}

LHA_Score.prototype = cbind(LHA_Score.prototype,levels(Data_Score.prototype[,7])) 
LHA_Score.prototype #First by ALPHABETICAL order

num_var=dim(LHA_Score.prototype)[1]
scores = as.numeric(LHA_Score.prototype[,1])
for(k in 1:length(scores))
	if(is.finite(scores[k])==FALSE)
		scores[k] = 0

LHA_Score.prototype=LHA_Score.prototype[sort(scores,index.return=TRUE)$ix,]
LHA_Score.prototype #Now by NUMERIC order
	

	
	
	
##########
#Validation of the complementary index
#########

#####
#Validation 1: The correlation of scores between the complementary and prototype index
cor(Score.complementary,Score.prototype,method="spearman")	
#####

#####
#Validation 2: The Spearman correlation of the complementary index scores and six proxy variables
#####
#Working with the necessary data, these are the six variables of interest
Data = Data_sub_All[,c(5,8:84)]
Data = scale(Data)
prototype = numeric()
prototype[1]=which(colnames(Data)==c("EDNO25"))
prototype[2]=which(colnames(Data)==c("UNEMP"))
prototype[3]=which(colnames(Data)==c("INCAV"))
prototype[4]=which(colnames(Data)==c("AVVAL"))
prototype[5]=which(colnames(Data)==c("OCCFM"))
prototype[6]=which(colnames(Data)==c("LONEPF"))

#The complementary index
cor(Score.complementary,Data[,prototype[1]],method="spearman")
cor(Score.complementary,Data[,prototype[2]],method="spearman")
cor(Score.complementary,Data[,prototype[3]],method="spearman")
cor(Score.complementary,Data[,prototype[4]],method="spearman")
cor(Score.complementary,Data[,prototype[5]],method="spearman")
cor(Score.complementary,Data[,prototype[6]],method="spearman")
#The prototype index
cor(Score.prototype,Data[,prototype[1]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[2]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[3]],,method=c("spearman"))
cor(Score.prototype,Data[,prototype[4]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[5]],method=c("spearman"))
cor(Score.prototype,Data[,prototype[6]],method=c("spearman"))




#####
#Validation 3: The INSPQ
####

#First set the working directory to where the Copy of EquivalenceTableCanada2006_A.csv table is:
setwd("C:/Users/kvincent/Desktop/MoH PCA project/MoH files/UBC Files/INSPQ")
INSPQ = read.csv("Copy of EquivalenceTableCanada2006_A.csv",header=TRUE)
INSPQ = INSPQ[46952:54422,]	#The relevant data
INSPQ = na.omit(INSPQ[,c(1,8,9)])	#Keep the first and second principal component scores

DA.Q = intersect(Data_Score.complementary[,2],INSPQ[,1]) #Which dissemination areas do we have INSPQ scores for?

Score.Q.complementary = numeric()
Score.Q.prototype = numeric()
INSPQ.M = numeric()
INSPQ.S = numeric()


for(k in DA.Q)
	{
	j = which(INSPQ[,1]==k)
			INSPQ.M = c(INSPQ.M,INSPQ[j,2])
			INSPQ.S = c(INSPQ.S,INSPQ[j,3])
	j = which(Data_Score.complementary[,2]==k)
			Score.Q.complementary = c(Score.Q.complementary,Data_Score.complementary[j,8])
	j = which(Data_Score.prototype[,2]==k)
			Score.Q.prototype = c(Score.Q.prototype,Data_Score.prototype[j,8])
	}
	
#complementary index scores
cor(INSPQ.M,Score.Q.complementary,method=c("spearman"))
cor(INSPQ.S,Score.Q.complementary,method=c("spearman"))

#prototypeindex scores scores
cor(INSPQ.M,Score.Q.prototype,method=c("spearman"))
cor(INSPQ.S,Score.Q.prototype,method=c("spearman"))
	

#####
#Validation 4: The pseudo-SEFI-like index
#####
#Working with the necessary data
Data = Data_sub_All[,c(5,8:84)]

#The variables of interest
SEFI.variables = numeric()
SEFI.variables[1]=which(colnames(Data)==c("X65PLUS"))
SEFI.variables[2]=which(colnames(Data)==c("POP15"))
SEFI.variables[3]=which(colnames(Data)==c("LONEPM"))
SEFI.variables[4]=which(colnames(Data)==c("LONEPF"))
SEFI.variables[5]=which(colnames(Data)==c("UNEMPF"))
SEFI.variables[6]=which(colnames(Data)==c("UNEMP15"))
SEFI.variables[7]=which(colnames(Data)==c("EDNO25"))

#Combing the related variables
Data.1 = Data[,SEFI.variables[1]]/(Data[,SEFI.variables[1]]+Data[,SEFI.variables[2]])
Data.2 = Data[,SEFI.variables[3]]+Data[,SEFI.variables[4]]
Data.3 = Data[,SEFI.variables[4]]
Data.4 = Data[,SEFI.variables[5]]
Data.5 = Data[,SEFI.variables[6]]
Data.6 = Data[,SEFI.variables[7]]

Data = cbind(Data.1,Data.2,Data.3,Data.4,Data.5,Data.6)

colnames(Data)=c("X65PLUS/(X65PLUS+POP15)","(LONEPM+LONEPF)","LONEPF","UNEMPF","UNEMP15","EDNO25")

Data = scale(Data)

#The first principal component
pData=princomp(Data,cor=TRUE,center=TRUE,scale=TRUE,cutoff=0,retx=TRUE)
weights.pca.1 = -1*loadings(pData)[,1]
weights.pca.1

#Now the Scores for the 6440 dissemination areas
Score.SEFI = numeric(6440)
for(k in 1:6440)
	Score.SEFI[k] = sum(Data[k,] * weights.pca.1)
	
#Standardize these values
Score.SEFI = (Score.SEFI - mean(Score.SEFI)) / sd(Score.SEFI)
Score.SEFI = 100*Score.SEFI + 1000

#The correlation of the complementary and SEFI index
cor(Score.complementary,Score.SEFI,method="spearman")

#The correlation of the prototype and SEFI index
cor(Score.prototype,Score.SEFI,method="spearman")
